import random
import string
import cherrypy
from tensorflow.keras.models import model_from_json
from skimage import transform
import numpy as np


class Service(object):

    def __init__(self):
        json_file = open(r'architecture.json', 'r')
        loaded_model_json = json_file.read()
        json_file.close()
        autoencoder = model_from_json(loaded_model_json)
        # load weights into new model
        autoencoder.load_weights(r"weights.h5")
        self.autoencoder = autoencoder
        self.img_size = 144
        self.threshold = 52.0
        self.std = 28.778374214746364 #0.05348464857296152
        self.mean = 97.78422013656773 #0.690545420997556 
        return

    def predict_def(self, img):
        img = np.asarray(img, dtype=np.uint8)        
        if img.shape[0] != self.img_size or img.shape[1] != self.img_size:
          img = transform.resize(img, (self.img_size, self.img_size),
                                 anti_aliasing=True)*255
        #X_test = img.flatten().reshape(1, self.img_size*self.img_size)
        X_test = np.array([img])

        X_test = (X_test - self.mean) / self.std        
        decoded = self.autoencoder.predict(X_test)
        score = np.linalg.norm(X_test[0] - decoded[0])
        if score > self.threshold:
            return "DEFECT, reconstruction_score: {}".format(score)
        return "NORMAL, reconstruction_score: {}".format(score)

    @cherrypy.expose
    @cherrypy.tools.json_in()
    def index(self):
        data = cherrypy.request.json
        return self.predict_def(data)


if __name__ == '__main__':
    cherrypy.quickstart(Service())
